@@ -1,6 +1,6 @@
 ;;; w3m-form.el --- Stuffs to handle <form> tag
 
-;; Copyright (C) 2001-2013 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2001-2014 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;          Yuuichi Teranishi  <teranisi@gohome.org>,
@@ -323,7 +323,7 @@ If no field in forward, return nil without moving."
 	  (setq bufs (cons (cons number (cons name value)) bufs))))
 	(setq plist (cddr plist))))
     (when bufs
-      (setq bufs (sort bufs (lambda (x y) (< (car x) (car y)))))
+      (setq bufs (sort bufs #'car-less-than-car))
       (if (eq (w3m-form-enctype form) 'multipart/form-data)
 	  (let ((boundary (apply 'format "--_%d_%d_%d" (current-time)))
 		file type)
@@ -354,7 +354,9 @@ If no field in forward, return nil without moving."
 			   "Content-Disposition: form-data; name=\""
 			   (car buf)
 			   "\"\r\n\r\n"
-			   (encode-coding-string (cdr buf) coding)
+			   (if (cdr buf)
+			       (encode-coding-string (cdr buf) coding)
+			     "")
 			   "\r\n"))
 		 (setq bufs (cdr bufs)))
 	       (insert "--" boundary "--\r\n")
@@ -955,8 +957,7 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
     (setq w3m-current-forms (if (eq w3m-type 'w3mmee)
 				forms
 			      (mapcar 'cdr
-				      (sort forms (lambda (x y)
-						    (< (car x)(car y)))))))
+				      (sort forms #'car-less-than-car))))
     (w3m-form-resume (or reuse-forms w3m-current-forms))))
 
 (defun w3m-form-replace (string &optional invisible)
@@ -1994,8 +1995,10 @@ selected rather than \(as usual\) some other window.  See
 			(if new-session
 			    'w3m-goto-url-new-session
 			  'w3m-goto-url))
-		      (concat (w3m-url-strip-query url)
-			      "?" (w3m-form-make-form-data form))))
+		      (let ((query (w3m-form-make-form-data form)))
+			(if (zerop (length query))
+			    url
+			  (concat (w3m-url-strip-query url) "?" query)))))
 	    (t
 	     (w3m-message "This form's method has not been supported: %s"
 			  (let (print-level print-length)
